﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace howto_wpf_find_control_by_name
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    public partial class Window1 : Window
    {
        public Window1()
        {
            InitializeComponent();
        }

        private void Button3_Click(object sender, RoutedEventArgs e)
        {
            // Find the object by name.
            string name = TextBox1.Text;
            DependencyObject descendant = FindDescendant(MainWindow, name);
            if (descendant == null)
            {
                MessageBox.Show("There such control " + name,
                    "No Such Control", MessageBoxButton.OK);
                return;
            }

            // Get the color by name.
            string color_name = TextBox2.Text;
            Color color;
            try
            {
                color = (Color)ColorConverter.ConvertFromString(color_name);
            }
            catch
            {
                MessageBox.Show("No such color " + color_name,
                    "No Such Color", MessageBoxButton.OK);
                return;
            }

            try
            {
                // Apply the color.
                // See if the object is a Panel.
                if (descendant is Panel)
                {
                    Panel panel = descendant as Panel;
                    panel.Background = new SolidColorBrush(color);
                }
                else if (descendant is Control)
                {
                    Control control = descendant as Control;
                    control.Background = new SolidColorBrush(color);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        // Find a descendant control by name.
        private static DependencyObject FindDescendant(
            DependencyObject parent, string name)
        {
            // See if this object has the target name.
            FrameworkElement element = parent as FrameworkElement;
            if ((element != null) && (element.Name == name)) return parent;

            // Recursively check the children.
            int num_children = VisualTreeHelper.GetChildrenCount(parent);
            for (int i = 0; i < num_children; i++)
            {
                // See if this child has the target name.
                DependencyObject child = VisualTreeHelper.GetChild(parent, i);
                DependencyObject descendant = FindDescendant(child, name);
                if (descendant != null) return descendant;
            }

            // We didn't find a descendant with the target name.
            return null;
        }
    }
}
